@@ -19,11 +19,9 @@
 
 #include <iomanip>
 
-#include "envt.hpp"
-#include "objects.hpp"
-#include "dinterpreter.hpp"
-#include "basic_pro.hpp"
-#include "nullgdl.hpp"
+//#include "envt.hpp"
+#include "dinterpreter.hpp" //calls envt
+#include "basic_pro.hpp" //for help_item
 
 #include <cassert> // always as last
 
@@ -210,7 +208,7 @@ EnvUDT::EnvUDT( ProgNodeP cN, BaseGDL* self,
 
   DSubUD* proUD=static_cast<DSubUD*>(pro);
 
-  forLoopInfo.InitSize( proUD->NForLoops());
+  forLoopInfo.InitSize(proUD->NForLoops());
 
   SizeT envSize;
   //   SizeT keySize;
@@ -266,7 +264,7 @@ EnvUDT::EnvUDT( BaseGDL* self, ProgNodeP cN, const string& parent, CallContext l
 
   DSubUD* proUD=static_cast<DSubUD*>(pro);
 
-  forLoopInfo.InitSize( proUD->NForLoops());
+  forLoopInfo.InitSize(proUD->NForLoops());
 
   SizeT envSize=proUD->var.size();
   parIx=proUD->key.size();
@@ -320,7 +318,7 @@ EnvUDT::EnvUDT( ProgNodeP callingNode_, DSubUD* newPro, DObjGDL** self):
 
   DSubUD* proUD= newPro; //static_cast<DSubUD*>(pro);
   
-  forLoopInfo.InitSize( proUD->NForLoops());
+  forLoopInfo.InitSize(proUD->NForLoops());
 
   SizeT envSize;
   //   SizeT keySize;
@@ -1161,7 +1159,7 @@ void EnvT::Catch()
   SizeT nParam = NParam();
   if( nParam == 0)
     {
-      if( KeywordSet( 0)) // CANCEL
+      if( KeywordSet("CANCEL"))
 	{
 	  caller->catchVar = NULL;
 	  caller->catchNode = NULL;
