@@ -32,7 +32,7 @@
 #include "calendar.hpp"
 
 //#define GDL_DEBUG
-#undef GDL_DEBUG
+//#undef GDL_DEBUG
 
 class DInterpreter;
 
@@ -160,6 +160,8 @@ public:
   bool KeywordSet( SizeT ix);
   //as KeyWordSet but return true if KW is not present/defined (for logical that default to true)
   bool BooleanKeywordAbsentOrSet( SizeT ix);
+  // this version is for testing boolean KWs: test if set (!null are ignored) but throws if not scalar
+  bool BooleanKeywordSet( SizeT ix);
 protected:
   // for HEAP_GC
   static void AddStruct( DPtrListT& ptrAccessible,  DPtrListT& objAccessible, 
@@ -258,6 +260,8 @@ public:
 
   // called after parameter definition
   void ResolveExtra();
+  BaseGDL* GetRefExtraList(DString &s);
+  BaseGDL** GetRefExtraListPtr(DString &s);
   friend class ExtraT;
 
   // used by compiler and from EnvT (for variable number of paramters)
@@ -281,7 +285,10 @@ public:
   EnvBaseT* Caller();
 
   // returns environment data, by value (but that by C++ reference)
-  BaseGDL*& GetKW(SizeT ix) { return env[ix];}
+  BaseGDL*& GetTheKW(SizeT ix) { return env[ix];}
+  BaseGDL* GetDefinedKW(SizeT ix);
+//  inline const BaseGDL* GetKW(SizeT ix) { return GetDefinedKW(ix);}
+  inline BaseGDL* GetKW(SizeT ix) { return GetDefinedKW(ix);}
 
   // used by HELP, SetNextPar(...)
   SizeT EnvSize() const { return env.size();}
@@ -337,14 +344,15 @@ public:
 
   void AssureGlobalKW( SizeT ix);
 
-  // converts parameter 'ix' if necessary and sets 'scalar' 
-  void AssureLongScalarPar( SizeT ix, DLong& scalar);
-  void AssureLongScalarPar( SizeT ix, DLong64& scalar);
+  // converts parameter 'ix' if necessary and sets 'scalar' . sets scalar to 0 if a type conversion was found, or throws depending on bool
+  void AssureLongScalarPar( SizeT ix, DLong& scalar, bool throwIfConversionErrorOccured=false);
+  void AssureLongScalarPar( SizeT ix, DLong64& scalar, bool throwIfConversionErrorOccured=false);
   // get i'th parameter
-  // throws if not defined (ie. never returns NULL)
-  BaseGDL*& GetParDefined(SizeT i); //, const std::string& subName = "");
+  // throws if not defined (ie. never returns NULL, but can optionally return a !NULL)
+  BaseGDL*& GetParDefined(SizeT i, bool rejectNulls=true); //, const std::string& subName = "");
 
   bool KeywordPresent( SizeT ix);
+  bool WriteableKeywordPresent( SizeT ix);
   bool KeywordPresentAndDefined( SizeT ix);
 
   void SetNextParUnckeckedVarNum(BaseGDL** arg1);
@@ -364,12 +372,14 @@ public:
 private:
   T* eArr;
   char buf[defaultLength * sizeof(T)]; // prevent constructor calls
+  SizeT currentMaxLength;
   SizeT sz;
 
 public:
 
   ForInfoListT(): eArr( reinterpret_cast<T*>(buf)), sz( 0)
-  {
+  { 
+    currentMaxLength=defaultLength;
   }
 
   ~ForInfoListT()
@@ -387,33 +397,40 @@ public:
   // must be called before access
   void InitSize( SizeT s)
   {
+//    std::cerr<<this<<": InitSize("<<s<<")\n";
     assert( sz == 0);
     if( s == 0)
       return;
-    sz = s;
-    if( s < defaultLength)
+    sz = 0;
+    if( s <= currentMaxLength) //initialise currentMaxLength objects
       {
-	for( SizeT i=0; i<s; ++i)
+	for( SizeT i=0; i<currentMaxLength; ++i)
 	  eArr[ i].Init();
 	return;
       }
     eArr = new T[ s]; // constructor called
+    currentMaxLength=s;
   }
   // only needed for EXECUTE
   void resize( SizeT s)
   {
-    if( s == sz)
+//    std::cerr<<this<<": resize("<<s<<"): ";
+    if( s == sz) {
+//    std::cerr<<" == "<<sz<<": return.\n";
       return;
+    }
     if( s < sz) // shrink
       {
+//      std::cerr << " < " << sz << ": shrink.\n";
 	for( SizeT i=s; i<sz; ++i)
 	  eArr[ i].ClearInit(); // in case eArr was allocated
 	sz = s;
 	return;
       }
     // s > sz -> grow
-    if( s <= defaultLength && eArr == reinterpret_cast<T*>(buf))
+    if( s <= currentMaxLength && eArr == reinterpret_cast<T*>(buf))
       {
+//      std::cerr << " > " << sz << " but <= currentMaxLength : grow.\n";
 	for( SizeT i=sz; i<s; ++i)
 	  eArr[ i].Init();
 	sz = s;
@@ -421,8 +438,10 @@ public:
       }
     // this should never happen (or only in extreme rarely cases)
     // the performance will go down
-    // s > defaultLength
+    // s > currentMaxLength
+//    std::cerr << " > " << sz << " and > currentMaxLength : should not happen.\n";
     T* newArr = new T[ s]; // ctor called
+    currentMaxLength=s;
     if( eArr != reinterpret_cast<T*>(buf))
       {
 	for( SizeT i=0; i<sz; ++i)
@@ -443,7 +462,7 @@ public:
     sz = s;
   }
   // T operator[]( SizeT i) const { assert( i<sz);  return eArr[i];}
-  T& operator[]( SizeT i) { assert( i<sz);  return eArr[i];}
+  T& operator[]( SizeT i) { assert( i<currentMaxLength);  return eArr[i];}
   SizeT size() const { return sz;}
   iterator begin() const { return &eArr[0];}
   iterator end() const { return &eArr[sz];}
@@ -455,7 +474,6 @@ public:
 };
 
 
-
 // for UD subroutines (written in GDL) ********************************
 class EnvUDT: public EnvBaseT
 {
@@ -473,8 +491,7 @@ public:
   };
 
 private:
-  ForInfoListT<ForLoopInfoT, 32> forLoopInfo;
-  // std::vector<ForLoopInfoT> forLoopInfo;
+   ForInfoListT<ForLoopInfoT, MAX_LOOPS_NUMBER> forLoopInfo;  //defaults to $MAIN$ values in dpro.hpp
 
   ProgNodeP         ioError; 
   DLong             onError; // on_error setting
@@ -615,8 +632,8 @@ public:
   BaseGDL*& GetPar( SizeT i);  
 
   // get i'th parameter
-  // throws if not defined (ie. never returns NULL)
-  BaseGDL*& GetParDefined(SizeT i); //, const std::string& subName = "");
+  // throws if not defined (ie. never returns NULL, but can optionally return a !NULL)
+  BaseGDL*& GetParDefined(SizeT i, bool rejectNulls=true); //, const std::string& subName = "");
 
   // throw for GDL_STRING, GDL_STRUCT, GDL_PTR and GDL_OBJ
   BaseGDL*& GetNumericParDefined( SizeT ix)
@@ -672,11 +689,21 @@ public:
     this->DeleteAtExit( res);
     return res;
   }
+  //same but ALWAYS a new copy, (BaseGDL::COPY) so it can be modified by the program 'safely' i.e. the original variable is not modified.
+  template <typename T> 
+  T* GetWriteableParAs( SizeT pIx)
+  {
+    BaseGDL* p = GetParDefined( pIx);
+    T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
+    this->DeleteAtExit( res);
+    return res;
+  }
+  
   // same as before for keywords
   template <typename T> 
   T* GetKWAs( SizeT ix)
   {
-    BaseGDL* p = GetKW( ix);
+    BaseGDL* p = GetKW( ix); //insure KW is not !NULL, which cannot be 'converted' 
     if( p == NULL)
       Throw( "Keyword is undefined: "+GetString( ix));
     if( p->Type() == T::t)
@@ -730,7 +757,8 @@ public:
   int KeywordIx( const std::string& k);
 
   // for use within library functions
-  // consider to use (note: 'static' is the point here):
+  // consider to use (note: 'static' is the point here, it MUST NOT be in a .h file 
+  // that is shared between multiple commands, unless the position of these KW is identical in libinit):
   // static int kwIx = env->KeywordIx( "KEYWORD");
   // bool kwSet = env->KeywordSet( kwIx);
   //
@@ -744,8 +772,9 @@ public:
   bool KeywordSet( SizeT ix);
   // GD added -- possibly very wrong?
   bool KeywordPresent( const std::string& kw);
-  bool KeywordPresent( SizeT ix)
-  { return EnvBaseT::KeywordPresent( ix);}
+  bool WriteableKeywordPresent( const std::string& kw);
+  inline bool KeywordPresent( SizeT ix) { return EnvBaseT::KeywordPresent( ix);}
+  inline bool WriteableKeywordPresent( SizeT ix){ return EnvBaseT::WriteableKeywordPresent( ix);}
 
   // local/global keyword/paramter
   bool LocalKW( SizeT ix) const
@@ -866,8 +895,8 @@ public:
   void AssureLongScalarKW( SizeT ix, DLong& scalar);
   void AssureLongScalarKW( SizeT ix, DLong64& scalar);
   // converts parameter 'ix' if necessary and sets 'scalar' 
-  void AssureLongScalarPar( SizeT ix, DLong& scalar);
-  void AssureLongScalarPar( SizeT ix, DLong64& scalar);
+  void AssureLongScalarPar( SizeT ix, DLong& scalar, bool throwIfConversionErrorOccured=false);
+  void AssureLongScalarPar( SizeT ix, DLong64& scalar, bool throwIfConversionErrorOccured=false);
 
   // same as for Long
   void AssureDoubleScalarKWIfPresent( const std::string& kw, DDouble& scalar);
