@@ -111,19 +111,15 @@ extern "C" {
 
 
 #include "dinterpreter.hpp"
-#include "datatypes.hpp"
-#include "envt.hpp"
-#include "dpro.hpp"
-
-
 #include "graphicsdevice.hpp"
 #include "dcommon.hpp"
 #include "dpro.hpp"
 
 #include "gdlhelp.hpp"
-#include "nullgdl.hpp"
 #include "terminfo.hpp"
-
+#ifndef _WIN32
+#include "shm_utils.hpp"
+#endif
 
 // for sorting compiled pro/fun lists by name
 struct CompFunName: public std::function<bool(DFun*, DFun*)>
@@ -527,24 +523,24 @@ static void help_lastmsg(EnvT* e)
   {
       // if LAST_MESSAGE is present, it is the only output.
       // All other kw are ignored *EXCEPT 'output'*.
-      BaseGDL** outputKW = NULL;
-
-	  DStructGDL* errorState = SysVar::Error_State();
-	  static unsigned msgTag = errorState->Desc()->TagIndex( "MSG");
-
-      static int outputIx = e->KeywordIx( "OUTPUT");
-	  if (e->WriteableKeywordPresent( outputIx)) 
-	    {    // Setup output return variable
-	      outputKW = &e->GetTheKW( outputIx);
-	      GDLDelete((*outputKW));
-	      *outputKW = static_cast<DStringGDL*>((errorState->GetTag( msgTag))
-	                                             ->Convert2( GDL_STRING, BaseGDL::COPY));
-	      return;
-			}
-			else {
-	    cout << (*static_cast<DStringGDL*>( errorState->GetTag( msgTag)))[0]<< '\n';
-	    return;
-	  }
+    BaseGDL** outputKW = NULL;
+    
+    DStructGDL* errorState = SysVar::Error_State();
+    static unsigned msgTag = errorState->Desc()->TagIndex( "MSG");
+    
+    static int outputIx = e->KeywordIx( "OUTPUT");
+    if (e->WriteableKeywordPresent( outputIx)) 
+      {    // Setup output return variable
+	outputKW = &e->GetTheKW( outputIx);
+	GDLDelete((*outputKW));
+	*outputKW = static_cast<DStringGDL*>((errorState->GetTag( msgTag))
+					     ->Convert2( GDL_STRING, BaseGDL::COPY));
+	return;
+      }
+    else {
+      cout << (*static_cast<DStringGDL*>( errorState->GetTag( msgTag)))[0]<< '\n';
+      return;
+    }
   }
     
   static DStringGDL* recall_commands_internal()
@@ -686,6 +682,9 @@ BaseGDL* recall_commands( EnvT* e)
       if (!doIndentation) ostr << "= ";
       if (par->IsAssoc())
         par->ToStream(ostr);
+#ifndef _WIN32
+	  else if (par->IsShared()) help_par_shared(par, ostr);
+#endif
     }
 
     // Dimension display
@@ -901,7 +900,16 @@ void help_help(EnvT* e)
 	  else SortAndPrintStream(ostr);
 	  return;
 	}
-
+    static int sharedIx = e->KeywordIx("SHARED_MEMORY");
+	if (e->KeywordSet(sharedIx)) {
+#ifndef _WIN32
+	  help_shared(e, *ostrp);
+	  if (briefKW) return;
+	  if (doOutput) (*outputKW) = StreamToGDLString(ostr, true);
+	  else SortAndPrintStream(ostr);
+#endif	  
+	  return;
+	}
 	static int namesIx = e->KeywordIx("NAMES");
 	bool isKWSetNames = e->KeywordPresent(namesIx);
 
@@ -1122,7 +1130,7 @@ void help_help(EnvT* e)
 	static int MEMORYIx = e->KeywordIx("MEMORY");
 	bool isKWSetMemory = e->KeywordSet(MEMORYIx);
 	static int PREFERENCESIx = e->KeywordIx("PREFERENCES");
-	bool isKWSetPreferences = e->KeywordSet("PREFERENCES");
+	bool isKWSetPreferences = e->KeywordSet(PREFERENCESIx);
 	if (isKWSetStructures) kw = true;
 
 	if ((isKWSetStructures or isKWSetRecall or isKWSetMemory or
