@@ -40,6 +40,7 @@
 #ifdef SIZEOF_SIZE_T
 #undef SIZEOF_SIZE_T
 #endif
+#include <list>
 #include <wx/wx.h>
 
 #include <wx/app.h>
@@ -50,7 +51,7 @@
 #include <wx/dragimag.h>
 #include <wx/dcbuffer.h>
 #include <wx/dnd.h>
-
+#include <wx/renderer.h>
 #include <wx/grid.h>
 #ifdef HAVE_WXWIDGETS_PROPERTYGRID
 //#include <wx/propgrid/propgrid.h>
@@ -70,10 +71,9 @@
 #include <deque>
 #include <map>
 
-#include "typedefs.hpp"
-#include "str.hpp"
-#include "datatypes.hpp"
+#include "envt.hpp"
 #include "widget.hpp"
+#include "basic_fun.hpp" //for GDL_TOSTRING
 
 #define gdlSCROLL_RATE 10
 #define gdlABSENT_SIZE_VALUE 15
@@ -111,6 +111,18 @@
 #define gdlSIZE_EVENT_HANDLER wxSizeEventHandler(gdlwxFrame::OnSizeWithTimer) //filter mouse events (manual resize) to avoid too many updtes for nothing
 #endif
 #define gdlSIZE_IMMEDIATE_EVENT_HANDLER wxSizeEventHandler(gdlwxFrame::OnSize) 
+
+
+
+#define UPDATE_VVALUE_HELPER(xxx,yyy) {\
+xxx* typed_old_gdl = static_cast<xxx*>(oldValue);\
+yyy* typed_old_value = static_cast<yyy*> (typed_old_gdl->DataAddr());\
+xxx* typed_new_gdl = static_cast<xxx*>(newValue);\
+yyy* typed_new_value = static_cast<yyy*> (typed_new_gdl->DataAddr());\
+updateVal<yyy>(typed_old_value,iold,typed_new_value,inew);\
+}
+
+
 typedef DLong WidgetIDT;
 static std::string widgetNameList[]={"BASE","BUTTON","SLIDER","TEXT","DRAW","LABEL","LIST","MBAR","DROPLIST","TABLE","TAB","TREE","COMBOBOX","PROPERTYSHEET","WINDOW"};
 static int    widgetTypeList[]={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14};
@@ -247,7 +259,7 @@ public:
 class wxAppGDL: public wxApp
 {
  wxGUIEventLoop loop;
-public:
+ public:
  int MyLoop();
 };
 
@@ -585,6 +597,18 @@ public:
    this->setFont();
    this->SetSensitive(sensitive);
 //   if (this->GetRealized()) this->RefreshWidget();
+    if (eventPro.size() > 0 ) {
+#ifdef GDL_DEBUG_WIDGETS
+      wxMessageOutputStderr().Printf(_T("Realize: SetEventPro: \"%s\" for %d\n"), eventPro, widgetID);
+#endif
+      bool found=GDLInterpreter::SearchCompilePro(eventPro, true); // true -> search for procedure
+    }
+    if (eventFun.size() > 0 ) {
+#ifdef GDL_DEBUG_WIDGETS
+      wxMessageOutputStderr().Printf(_T("Realize: SetEventFun: \"%s\" for %d\n"), eventFun, widgetID);
+#endif
+      bool found=GDLInterpreter::SearchCompilePro(eventFun, false); // false -> search for function
+    }
    if( notifyRealize != "") { //insure it is called once only for this.
       std::string note=notifyRealize;
       notifyRealize.clear();
@@ -676,9 +700,27 @@ public:
   
   virtual bool IsUsingInteractiveEventLoop() {/*std::cerr<<"IsEventLoopBlocked on a not-top widget, please report."<<std::endl*/;return false;} //default for a normal widget
 
-  void SetEventPro( const DString& ePro) { eventPro = StrUpCase( ePro);}
+  void SetEventPro( const DString& ePro) {
+    eventPro = StrUpCase( ePro);
+#ifdef GDL_DEBUG_WIDGETS
+      wxMessageOutputStderr().Printf(_T("SetEventPro: \"%s\" for %d\n"), eventPro, widgetID);
+#endif
+    if (eventPro.size() > 0) {
+      bool found = GDLInterpreter::SearchCompilePro(eventPro, true); // true -> search for procedure
+    }
+   }
+  
   const DString& GetEventPro() const { return eventPro;};
-  void SetEventFun( const DString& eFun) { eventFun = StrUpCase( eFun);}
+  
+  void SetEventFun( const DString& eFun) {
+    eventFun = StrUpCase(eFun);
+#ifdef GDL_DEBUG_WIDGETS
+    wxMessageOutputStderr().Printf(_T("SetEventFun: \"%s\" for %d\n"),eventFun, widgetID);
+#endif
+    if (eventFun.size() > 0) {
+      bool found = GDLInterpreter::SearchCompilePro(eventFun, false); // false -> search for function
+    }
+  }
   const DString& GetEventFun() const { return eventFun;}
   void SetNotifyRealize( const DString& eNR) { notifyRealize = StrUpCase( eNR);}
   const DString& GetNotifyRealize() const { return notifyRealize;}
@@ -1589,7 +1631,6 @@ class GDLWidgetTable: public GDLWidget
 //  DLong tabMode;
   DLong x_scroll_size_columns;
   DLong y_scroll_size_rows;
-  DStringGDL * valueAsStrings;
   bool         updating; //widget is modified by program (avoid sending events)
 
 public:
@@ -1619,20 +1660,21 @@ public:
 		  BaseGDL* value_,
 		  DLong xScrollSize_,
 		  DLong yScrollSize_,
-                  DStringGDL* valueAsStrings_,
-                  DULong eventFlags_
-         );
+      DStringGDL* valueAsStrings,
+      DULong eventFlags_
+  );
 
 ~GDLWidgetTable();
-
-
+  std::vector<int> GetSortedSelectedRowsOrColsList(DLongGDL* selection, bool doCol);
+  DStringGDL* GetCurrentFormat(){return format;}
+  void SetCurrentFormat(DStringGDL* f){format=f;}
   int  GetMajority(){return majority;}
   bool IsTable() const final { return true;}
   void SetDOW(DStringGDL* val){GDLDelete(daysOfWeek); daysOfWeek=val->Dup();}
   void SetAmPm(DStringGDL* val){GDLDelete(amPm); amPm=val->Dup();};
   void SetMonth(DStringGDL* val){GDLDelete(month); month=val->Dup();};
 
-  DLongGDL* GetSelection();
+  DLongGDL* GetSelection(bool dothrow=false);
   
   void SetAlignment(DByteGDL* val){GDLDelete(table_alignment); table_alignment=val->Dup();};
   void DoAlign();
@@ -1669,21 +1711,67 @@ public:
   void DeleteColumns(DLongGDL* selection=NULL);
   void DeleteRows(DLongGDL* selection=NULL);
 
-  bool InsertColumns(DLong count, DLongGDL* selection=NULL);
-  bool InsertRows(DLong count, DLongGDL* selection=NULL);
+  bool InsertColumns(DLong count, bool insertAtEnd, DLongGDL* selection=NULL);
+  bool InsertRows(DLong count, bool insertAtEnd, DLongGDL* selection=NULL);
 
   void SetSelection(DLongGDL* selection);
-  DStringGDL* GetTableValues(DLongGDL* selection=NULL);
-  BaseGDL* GetTableValuesAsStruct(DLongGDL* selection=NULL);
-  void SetTableValues(DStringGDL *val, DLongGDL* selection=NULL);
+
+  int GetDataNCols(){
+    if ( majority == NONE_MAJOR) return vValue->Dim(0);
+    else if ( majority == COLUMN_MAJOR) return vValue->Dim(0);
+    else  return static_cast<DStructGDL*>(vValue)->NTags();
+  }
+  int GetDataNRows(){
+    if ( majority == NONE_MAJOR) return vValue->Dim(1);
+    else if ( majority == COLUMN_MAJOR) return static_cast<DStructGDL*>(vValue)->NTags();
+    else  return vValue->Dim(0);
+  }
+  template<typename T1, typename T2>
+  DString GetRawEditingValue(T1* value, const int nEl, const int n) {
+    if (n > nEl-1) return "";
+    T2* val=static_cast<T2*>(value->DataAddr());
+    std::stringstream os;
+    os << val[n];
+    return os.str();
+  }
+  template<typename T1, typename T2>
+  DString SetEditedValue(wxString s, T1* value, const int nEl, const int n, DStringGDL* format) {
+    if (n > nEl-1) return "";
+    //convert to STRING using FORMAT.
+    T2* val=static_cast<T2*>(value->DataAddr());
+    std::stringstream os;
+    os << s;
+    os >> val[n];
+    T1* subvalue=new T1(val[n]);
+    Guard<T1> guard(subvalue);
+    return (*CallStringFunction(subvalue, format))[0];
+  }
+  void UpdatevValues(BaseGDL* oldValue, SizeT iold, BaseGDL* newValue, SizeT inew);
+
+template <typename T>
+void updateVal(T* oldvalue, SizeT iold, T* newvalue, SizeT inew){
+    oldvalue[iold]=newvalue[inew];
+  }
+  
+  template <typename T1, typename T2>
+  void PopulateWithSelection(T1* res, int colTL, int colBR, int rowTL, int rowBR);
+  BaseGDL* GetSelectionValuesForStructs(int colTL, int colBR, int rowTL, int rowBR);
+  BaseGDL* GetSelectionValues(int colTL, int colBR, int rowTL, int rowBR);
+  BaseGDL* GetDisjointSelectionValues(DLongGDL* selection);
+  BaseGDL* GetDisjointSelectionValuesForStructs(DLongGDL* selection);
+  template <typename T1, typename T2>
+  void PopulateWithDisjointSelection(T1* resGDL, DLongGDL* selection);
+  BaseGDL* GetTableValues(DLongGDL* selection=NULL);
+  void SetTableValues(BaseGDL* value, DStringGDL *stringval, DLongGDL* selection=NULL);
   void SetValue(BaseGDL * val){GDLDelete(vValue); vValue=val->Dup();};
   
   void SetTableView(DLongGDL* pos);
-  void EditCell(DLongGDL* pos);
-  void SetTableNumberOfColumns( DLong ncols);
-  void SetTableNumberOfRows( DLong nrows);
+  void MakeCellEditable(DLongGDL* pos);
+  void SetTableXsizeAsNumberOfColumns( DLong ncols);
+  void SetTableYsizeAsNumberOfRows( DLong nrows);
   
   bool IsSomethingSelected();
+  bool GetValidTableSelection(DLongGDL* &selection);
   
   bool IsUpdating(){return updating;}
   void ClearUpdating(){updating=false;}
@@ -1885,6 +1973,9 @@ class GDLWidgetSlider: public GDLWidget
   DLong minimum; 
   DLong maximum;
   DString title;
+
+  wxSlider* GetSliderWidget();
+
 public:
   GDLWidgetSlider( WidgetIDT parentID, EnvT* e, DLong value_,
        DULong eventFlags_ ,
@@ -1923,7 +2014,7 @@ public:
 	    const wxString& name = wxPanelNameStr):
   wxGrid( container, id, pos, size, style, name )
   , GDLWidgetTableID(id)
-  {
+{
    //replaced by addtodesiredevents.
 //    Connect(id,wxEVT_GRID_COL_SIZE,wxGridSizeEventHandler(wxGridGDL::OnTableColResizing));
 //    Connect(id,wxEVT_GRID_ROW_SIZE,wxGridSizeEventHandler(wxGridGDL::OnTableRowResizing));
@@ -1934,7 +2025,11 @@ public:
   ~wxGridGDL(){
 #ifdef GDL_DEBUG_WIDGETS
     std::cout << "~wxGridGDL: " << this << std::endl;
-#endif 
+#endif
+  }
+
+  wxWindowID GetWidgetTableID() {
+    return GDLWidgetTableID;
   }
   
   bool IsSomethingSelected(){
@@ -1948,13 +2043,19 @@ public:
       if ( selectionCol.GetCount() > 0 ) return true;
       return false;
   }
+  static bool is_unique_wxPoint (wxPoint first, wxPoint second)
+{ return ( first.x==second.x && first.y==second.y ); }
+  static bool compare_wxPoint (wxPoint first, wxPoint second)
+{ if (first.x==second.x) return ( first.y<second.y );
+  else return ( first.x<second.x ); 
+  }
   std::vector<wxPoint> GetSelectedDisjointCellsList(){
-      std::vector<wxPoint> list;
+      std::list<wxPoint> mylist;
       wxGridCellCoordsArray cellSelection=this->GetSelectedCells();
       for( int i=0; i<cellSelection.Count(); i++ ) {
        int row = cellSelection[i].GetRow();
        int col = cellSelection[i].GetCol();
-       list.push_back(wxPoint(row,col));
+       mylist.push_back(wxPoint(row,col));
       }
 
       wxGridCellCoordsArray selectionTL=this->GetSelectionBlockTopLeft();
@@ -1966,19 +2067,23 @@ public:
        int colBR = selectionBR[k].GetCol();
        int nrows=rowBR-rowTL+1;
        int ncols=colBR-colTL+1;
-       for ( int i=0; i< nrows; ++i) for (int j=0; j<ncols; ++j) list.push_back(wxPoint(rowTL+i,colTL+j));
+       for ( int i=0; i< nrows; ++i) for (int j=0; j<ncols; ++j) mylist.push_back(wxPoint(rowTL+i,colTL+j));
       }
       wxArrayInt selectionRow=this->GetSelectedRows();
       for( int k=0; k<selectionRow.GetCount(); k++ ) {
        int row = selectionRow[k];
-       for ( int i=0; i< this->GetNumberCols(); ++i) list.push_back(wxPoint(row,i));
+       for ( int i=0; i< this->GetNumberCols(); ++i) mylist.push_back(wxPoint(row,i));
       }
       wxArrayInt selectionCol=this->GetSelectedCols();
       for( int k=0; k<selectionCol.GetCount(); k++ ) {
        int col = selectionCol[k];
-       for ( int i=0; i< this->GetNumberRows(); ++i) list.push_back(wxPoint(i,col));
-      }      
-      return list;
+       for ( int i=0; i< this->GetNumberRows(); ++i) mylist.push_back(wxPoint(i,col));
+      }
+      mylist.sort(compare_wxPoint);
+      mylist.unique(is_unique_wxPoint);
+      std::vector<wxPoint> ret;
+      for (std::list<wxPoint>::iterator it = mylist.begin(); it != mylist.end(); ++it) ret.push_back(*it);
+      return ret;
   }
 
   wxArrayInt GetSelectedBlockOfCells() {
@@ -2028,47 +2133,13 @@ public:
     return block;
   }
   
-   wxArrayInt GetSortedSelectedColsList(){
-   std::vector<wxPoint> list=GetSelectedDisjointCellsList();
-   wxArrayInt cols;
-   if (list.empty()) return cols; 
-   std::vector<wxPoint>::iterator iPoint;
-   std::vector<int> allCols;
-   std::vector<int>::iterator iter;
-   for ( iPoint = list.begin(); iPoint !=list.end(); ++iPoint) {
-       allCols.push_back((*iPoint).y);
-    }
-   std::sort (allCols.begin(), allCols.end());
-   int theCol=-1;
-   for ( iter = allCols.begin(); iter !=allCols.end(); ++iter) {
-       if ((*iter)!=theCol) {theCol=(*iter);cols.Add(theCol);}
-    }
-   return cols;
-  }
-  wxArrayInt GetSortedSelectedRowsList(){
-   std::vector<wxPoint> list=GetSelectedDisjointCellsList();
-   wxArrayInt rows;
-   if (list.empty()) return rows; 
-   std::vector<wxPoint>::iterator iPoint;
-   std::vector<int> allRows;
-   std::vector<int>::iterator iter;
-   for ( iPoint = list.begin(); iPoint !=list.end(); ++iPoint) {
-       allRows.push_back((*iPoint).x);
-    }
-   std::sort (allRows.begin(), allRows.end());
-   int theRow=-1;
-   for ( iter = allRows.begin(); iter !=allRows.end(); ++iter) {
-       if ((*iter)!=theRow) {theRow=(*iter);rows.Add(theRow);}
-    }
-   return rows;
-  }
-
   void OnTableCellSelection(wxGridEvent & event);
   void OnTableRangeSelection(wxGridRangeSelectEvent & event);
   void OnTableColResizing(wxGridSizeEvent & event);
   void OnTableRowResizing(wxGridSizeEvent & event); 
 //  void OnText( wxCommandEvent& event);
-//  void OnTextEnter( wxCommandEvent& event);
+//  void OnTextChanging( wxGridEvent & event);
+  void OnTextChanged( wxGridEvent & event);
 };
 #ifdef HAVE_WXWIDGETS_PROPERTYGRID
 //
@@ -2233,6 +2304,7 @@ public:
  void OnMouseMove(wxMouseEvent& event);
  void OnMouseDown(wxMouseEvent& event);
  void OnMouseUp(wxMouseEvent& event);
+ void OnMouseDownDble(wxMouseEvent& event);
  void OnMouseWheel(wxMouseEvent& event);
  void OnKey(wxKeyEvent& event);
  void OnFakeDropFileEvent(wxDropFilesEvent& event);
@@ -2240,7 +2312,7 @@ public:
 // void OnPlotWindowSize(wxSizeEvent &event);
 
 };
-
+DStringGDL* CallStringFunction(BaseGDL* val, BaseGDL* format);
 #endif
 
 #endif
