@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 #include "includefirst.hpp"
+#include "version.hpp"
 
 #ifndef _WIN32
 #include <sys/utsname.h>
@@ -28,13 +29,13 @@
 #include <omp.h> // for !CPU
 #endif
 
-#include "nullgdl.hpp"
+//#include "nullgdl.hpp"
 
 #include "objects.hpp"
 #include "dstructgdl.hpp"
 #include "graphicsdevice.hpp"
 
-#include "file.hpp"
+#include "file.hpp" //for ExpandPath
 
 #ifdef _MSC_VER
 #include "gtdhelper.hpp"
@@ -398,7 +399,11 @@ namespace SysVar
     DVar* sysVarList_warnIx = sysVarList[ warnIx];
     return static_cast<DStructGDL*>(sysVarList_warnIx->Data());
   }
-
+  DIntGDL* Except()
+  {
+    DVar* sysVarList_exceptIx = sysVarList[ exceptIx];
+    return static_cast<DIntGDL*>(sysVarList_exceptIx->Data());
+  }
   DString MsgPrefix()
   {
     DStructGDL* errorState = Error_State();
@@ -651,7 +656,7 @@ namespace SysVar
 
     // !GDL (to allow distinguish IDL/GDL with DEFSYSV, '!gdl', exists=exists )
     DStructGDL*  gdlStruct = new DStructGDL( "!GNUDATALANGUAGE");
-    gdlStruct->NewTag("RELEASE", new DStringGDL( VERSION));
+    gdlStruct->NewTag("RELEASE", new DStringGDL( VERSION + (VERSION[0] == 'v' ? 1 : 0)));
 
     // creating an explicit build date in !GDL (also exist in !version)
     gdlStruct->NewTag("BUILD_DATE", new DStringGDL(BUILD_DATE));
