@@ -57,6 +57,9 @@
 
 #include "grib.hpp"
 #include "semshm.hpp"
+#ifndef _WIN32
+#include "shm.hpp"
+#endif
 #include "labelregion.hpp"
 #include "value_locate.hpp"
 
@@ -312,9 +315,8 @@ void LibInit()
 			  "INTERNAL_LIB_GDL","KEYS","LAST_MESSAGE","LIB","MEMORY","NAMES",
 		"OBJECTS","OUTPUT","PATH_CACHE","PREFERENCES","PROCEDURES",
 			  "RECALL_COMMANDS","ROUTINES","SOURCE_FILES","STRUCTURES",
-              "SYSTEM_VARIABLES","TRACEBACK", "COMMON","LEVEL", KLISTEND};
-  const string helpWarnKey[]={"BREAKPOINTS","DLM", "MESSAGES",
-			      "SHARED_MEMORY", KLISTEND};
+              "SYSTEM_VARIABLES","TRACEBACK", "COMMON","LEVEL", "SHARED_MEMORY", KLISTEND};
+  const string helpWarnKey[]={"BREAKPOINTS","DLM", "MESSAGES", KLISTEND};
   new DLibPro(lib::help_pro,string("HELP"),-1,helpKey,helpWarnKey);
 
   new DLibPro(lib::delvar_pro,string("DELVAR"),-1,NULL,NULL);
@@ -360,7 +362,10 @@ void LibInit()
 
   const string stringKey[]={COMMONKEYWORDSFORSTRINGFORMATTINGWITHIMPLIEDPRINT,"PRINT",KLISTEND};
   new DLibFun(lib::string_fun,string("STRING"),-1,stringKey);
-
+  //idem but ony with "FORMAT" option, for internal use
+  const string gdl_stringKey[]={"FORMAT",KLISTEND};
+  new DLibFun(lib::gdl_tostring_fun,string("GDL_TOSTRING"),1,gdl_stringKey);
+  
   const string defsysvKey[]={"EXISTS","TEST",KLISTEND};
   new DLibPro(lib::defsysv,string("DEFSYSV"),4,defsysvKey); 
 
@@ -384,13 +389,14 @@ void LibInit()
   new DLibFunRetNew(lib::ulon64arr,string("ULON64ARR"),MAXRANK,arrKey,NULL,true);
   new DLibFunRetNew(lib::fltarr,string("FLTARR"),MAXRANK,arrKey,NULL,true);
   new DLibFunRetNew(lib::dblarr,string("DBLARR"),MAXRANK,arrKey,NULL,true);
-  new DLibFunRetNew(lib::strarr,string("STRARR"),MAXRANK,arrKey,NULL,true);
   new DLibFunRetNew(lib::complexarr,string("COMPLEXARR"),MAXRANK,arrKey,NULL,true);
   new DLibFunRetNew(lib::dcomplexarr,string("DCOMPLEXARR"),MAXRANK,arrKey,NULL,true);
 
-  const string ptrArrKey[]={"ALLOCATE_HEAP",KLISTEND};
-  new DLibFunRetNew(lib::ptrarr,string("PTRARR"),MAXRANK,ptrArrKey);
-  new DLibFunRetNew(lib::objarr,string("OBJARR"),MAXRANK);
+  new DLibFunRetNew(lib::strarr,string("STRARR"),MAXRANK,NULL,NULL,true);
+
+const string ptrArrKey[]={"ALLOCATE_HEAP","NOZERO",KLISTEND};
+  new DLibFunRetNew(lib::ptrarr,string("PTRARR"),MAXRANK,ptrArrKey,NULL,true);
+  new DLibFunRetNew(lib::objarr,string("OBJARR"),MAXRANK,arrKey,NULL,true);
 
   const string ptr_newKey[]={"NO_COPY","ALLOCATE_HEAP",KLISTEND};
   new DLibFunRetNew(lib::ptr_new,string("PTR_NEW"),1,ptr_newKey);
@@ -1019,10 +1025,10 @@ void LibInit()
   new DLibFunRetNew(lib::ll_arc_distance, string("LL_ARC_DISTANCE"), 3, ll_arc_distanceKey);
   const string command_line_argsKey[] = {"COUNT","RESET","SET", KLISTEND }; //note: reset and set are unsupported extensions.
 // original  const string command_line_argsKey[] = {"COUNT", KLISTEND };
-  new DLibFunRetNew(lib::command_line_args_fun, string("COMMAND_LINE_ARGS"), 0, command_line_argsKey);
+new DLibFunRetNew(lib::command_line_args_fun, string("COMMAND_LINE_ARGS"), 0, command_line_argsKey);
 
- // const string pmKey[] = {"FORMAT", "TITLE", KLISTEND };
-  new DLibPro(lib::pm, string("PM"), -1);
+const string pmKey[] = {"FORMAT", "TITLE", KLISTEND };
+new DLibPro(lib::pm_pro, string("PM"), -1, pmKey);
 
   const string constantKey[] = {"DOUBLE", KLISTEND };
   new DLibFunRetNew(lib::constant, string("IMSL_CONSTANT"), 2, constantKey);
@@ -1079,5 +1085,20 @@ void LibInit()
 
   const string gaussfitKey[] = {"CHISQ","ESTIMATES", "MEASURE_ERRORS", "NTERMS", "SIGMA", "YERROR",KLISTEND};
   new DLibFunRetNew(lib::gaussfit, string("GAUSSFIT"), 3, gaussfitKey);
+  #ifndef _WIN32
+  //SHM --------------------------------------------------------------------------------------
+  // Note: counted values, do not change order:
+  const string shmmapKey[] = {
+	"BYTE","COMPLEX","DCOMPLEX","DOUBLE","FLOAT","INTEGER","L64","LONG","UINT","UL64","ULONG", "DIMENSION","SIZE", "TEMPLATE", "TYPE", //"TYPE": position 14
+	//the rest free
+             "DESTROY_SEGMENT","FILENAME","GET_NAME", "GET_OS_HANDLE", "OFFSET", 
+              "OS_HANDLE", "PRIVATE", "SYSV", KLISTEND};
+  new DLibPro(lib::shmmap_pro,string("SHMMAP"),9,shmmapKey);
+  new DLibPro(lib::shmunmap_pro,string("SHMUNMAP"),1);
+  // Note: counted values, do not change order:
+  const string shmvarKey[] = {"BYTE","COMPLEX","DCOMPLEX","DOUBLE","FLOAT","INTEGER","L64","LONG","UINT","UL64","ULONG","DIMENSION","SIZE", "TEMPLATE", "TYPE",KLISTEND};
+  new DLibFunRetNew(lib::shmvar_fun,string("SHMVAR"),9,shmvarKey);
+  new DLibFunRetNew(lib::shmdebug_fun,string("SHMDEBUG"),1);
+  #endif
 }
 
