@@ -44,13 +44,11 @@
 #include "fftw.hpp"
 #endif
 
-#if defined(USE_EIGEN)
 #include "matrix_cholesky.hpp"
 #include "least_squares.hpp"
-#endif
-
 #include "matrix_invert.hpp"
 #include "triangulation.hpp"
+#include "eigenvalues_solvers.hpp"
 
 #ifdef HAVE_QHULL
 #include "qhull.hpp"
@@ -86,7 +84,7 @@ void LibInit_jmg()
   // Since we have a difference between the IDL way and the Eigen way
   // we (temporary) remove these 2 codes in the CVS of GDL
   // Help welcome. 
-#if defined(USE_EIGEN)
+  //#if defined(USE_EIGEN)
   const string cholKey[]={"DOUBLE",KLISTEND};
   new DLibPro(lib::choldc_pro,string("CHOLDC"),2,cholKey);
   new DLibFunRetNew(lib::cholsol_fun,string("CHOLSOL"),3,cholKey);
@@ -96,13 +94,38 @@ void LibInit_jmg()
   new DLibPro(lib::la_choldc_pro,string("LA_CHOLDC"),1,lacholdcKey);
   const string lacholsolKey[]={"DOUBLE","STATUS",KLISTEND};
   new DLibFunRetNew(lib::la_cholsol_fun,string("LA_CHOLSOL"),2,lacholsolKey);
+  //#endif
 
   // Ilia N. & Alain C., Summer 2015
   const string laleastsquaresKey[]={"DOUBLE","METHOD","RANK","RCONDITION","RESIDUAL","STATUS",KLISTEND};
   new DLibFunRetNew(lib::la_least_squares_fun,string("LA_LEAST_SQUARES"),2,laleastsquaresKey);
 
-#endif
+  // square matrices , eispack 'old' free code, by GD.
+  
+  const string elmhesKey[]={"COLUMN", "DOUBLE", "NO_BALANCE",KLISTEND};
+  new DLibFunRetNew(lib::elmhes_fun,string("ELMHES"),1,elmhesKey);
 
+  // AC will return unavailability when compiled without Eigen
+  const string la_elmhesKey[]={"DOUBLE",KLISTEND};
+  const string la_elmhesWarnKey[]={"BALANCE","NORM_BALANCE","PERMUTE_RESULT","SCALE_RESULT",KLISTEND};
+  new DLibFunRetNew(lib::la_elmhes_fun,string("LA_ELMHES"),2,la_elmhesKey,la_elmhesWarnKey);
+
+  const string hqrKey[]={"COLUMN","DOUBLE",KLISTEND};
+  new DLibFunRetNew(lib::hqr_fun,string("HQR"),1,hqrKey);
+  
+  //GD: replaced la_trired from gsl by la_trired from eigen (if eigen is present) as it gives the same results as IDL's LA_TRIRED and is 5 times faster.
+  // AC: but if GDL compiled without Eigen, GSL version still here (see below) 
+#if defined(USE_EIGEN)
+  const string la_trired1Key[]={"DOUBLE","DEBUG",KLISTEND};
+  const string la_trired1WarnKey[]={"UPPER",KLISTEND};
+  new DLibPro(lib::la_trired_pro,string("LA_TRIRED"),3,la_trired1Key,la_trired1WarnKey);
+#endif
+  
+  const string triredKey[]={"DOUBLE",KLISTEND};
+  new DLibPro(lib::trired_pro,string("TRIRED"),3,triredKey);
+  const string triqlKey[]={"DOUBLE",KLISTEND};
+  new DLibPro(lib::triql_pro,string("TRIQL"),3,triqlKey);
+    
 #if defined(HAVE_LIBGSL) && defined(HAVE_LIBGSLCBLAS)
   
   const string invertKey[]={"DOUBLE","GSL","EIGEN",KLISTEND};
@@ -136,8 +159,11 @@ void LibInit_jmg()
   const string interpolateKey[]={"CUBIC","DOUBLE","GRID","MISSING","NEAREST_NEIGHBOUR",KLISTEND};
   new DLibFunRetNewTP(lib::interpolate_fun,string("INTERPOLATE"),4,interpolateKey);  //UsesThreadPOOL 
 
-  const string la_triredKey[]={"DOUBLE","UPPER",KLISTEND};
+  //GD: replaced la_trired from gsl by la_trired from eigen (if eigen is present) as it gives the same results as IDL's LA_TRIRED and is 5 times faster.
+#if  !defined(USE_EIGEN)
+  const string la_triredKey[]={"DOUBLE","UPPER","DEBUG",KLISTEND};
   new DLibPro(lib::la_trired_pro,string("LA_TRIRED"),3,la_triredKey);
+#endif
 #endif
 
   const string macharKey[]={"DOUBLE",KLISTEND};
@@ -322,10 +348,10 @@ void LibInit_jmg()
 
   // call_external (by Christoph Fuchs)
   const string call_externalKey[] = {"VALUE", "ALL_VALUE", "RETURN_TYPE",
-               "B_VALUE", "I_VALUE", "L_VALUE", "F_VALUE", "D_VALUE",
-               "UI_VALUE", "UL_VALUE", "L64_VALUE", "UL64_VALUE", "S_VALUE",
-               "UNLOAD", "ALL_GDL", "STRUCT_ALIGN_BYTES"
-    , "DEFAULT", "PORTABLE", "VAX_FLOAT" // obsoleted VMS
+    "B_VALUE", "I_VALUE", "L_VALUE", "F_VALUE", "D_VALUE",
+    "UI_VALUE", "UL_VALUE", "L64_VALUE", "UL64_VALUE", "S_VALUE",
+    "UNLOAD", "ALL_GDL", "STRUCT_ALIGN_BYTES","CDECL",
+    "DEFAULT", "PORTABLE", "VAX_FLOAT" // obsoleted VMS
     , KLISTEND };
   new DLibFunRetNew(lib::call_external, string("CALL_EXTERNAL"), -1, call_externalKey);
 }
