@@ -20,14 +20,10 @@
 #include <iostream>
 #include <complex>
 #include <cmath>
-
-#include "datatypes.hpp"
+#include <cfenv>
 #include "envt.hpp"
-#include "dinterpreter.hpp"
-#include "initsysvar.hpp"
 #include "math_utl.hpp"
 #include "math_fun_jmg.hpp"
-#include "graphicsdevice.hpp"
 #include "gdl_util.hpp"
 
 //#define GDL_DEBUG
@@ -59,8 +55,10 @@ namespace lib {
     long int ibeta, it, irnd, ngrd, machep, negep, iexp, minexp, maxexp;
     float  eps, epsneg, xmin, xmax;
     double epsD, epsnegD, xminD, xmaxD;
-    
-    if( e->KeywordSet(0)) //DOUBLE
+
+    static int doubleIx=e->KeywordIx("DOUBLE");
+
+    if (e->KeywordSet(doubleIx))
       {
 	machar_d(&ibeta, &it, &irnd, &ngrd, &machep, 
 		 &negep, &iexp, &minexp, &maxexp, 
