@@ -23,21 +23,16 @@
 #endif
 
 #ifdef USE_NETCDF
-#  include "includefirst.hpp"
 #  define HDF 1
 
 #  include <netcdf.h>
 
-#  include "datatypes.hpp"
 #  include "envt.hpp"
-
-//#  include "dpro.hpp"
-//#  include "dinterpreter.hpp"
 #  include "ncdf_cl.hpp"
 
 
 //#  define GDL_DEBUG
-#  undef GDL_DEBUG
+//#  undef GDL_DEBUG
 
 namespace lib {
 
@@ -218,7 +213,7 @@ namespace lib {
     ncdf_handle_error(e,status,"NCDF_ATTGET");
 
     if (att_type == NC_CHAR) {
-      DByteGDL* temp = new DByteGDL(dimension(length));
+      DByteGDL* temp = new DByteGDL(dimension(length),BaseGDL::NOZERO);
       status = nc_get_att_text(cdfid, varid, attname.c_str(), (char*)(&((*temp)[0])));
       ncdf_handle_error(e, status, "NCDF_ATTGET");
       delete e->GetParGlobal(nParam-1);
