@@ -29,16 +29,10 @@
 #include <fstream>
 #include <iostream>
 
-#include "datatypes.hpp"
-#include "envt.hpp"
-#include "dpro.hpp"
 #include "dinterpreter.hpp"
 
 #include "ncdf_cl.hpp"
 
-//#define GDL_DEBUG
-//#undef GDL_DEBUG
-
 namespace lib {
 
   using namespace std;
